@@ -6,7 +6,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
+#  Copyright (c) 1994 - 99	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -34,10 +34,10 @@
 #include "RubikP.h"
 #include "Rubik3dP.h"
 
-static void InitializeRubik3D(Widget request, Widget new);
-static void ExposeRubik3D(Widget new, XEvent * event, Region region);
+static void InitializeRubik3D(Widget request, Widget renew);
+static void ExposeRubik3D(Widget renew, XEvent * event, Region region);
 static void ResizeRubik3D(Rubik3DWidget w);
-static Boolean SetValuesRubik3D(Widget current, Widget request, Widget new);
+static Boolean SetValuesRubik3D(Widget current, Widget request, Widget renew);
 static void MoveRubik3DTl(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
 static void MoveRubik3DTop(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
 static void MoveRubik3DTr(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
@@ -100,8 +100,11 @@ static char defaultTranslationsRubik3D[] =
    <KeyPress>w: Write()\n\
    <KeyPress>u: Undo()\n\
    <KeyPress>s: Solve()\n\
-   <KeyPress>i: Increment()\n\
    <KeyPress>d: Decrement()\n\
+   <KeyPress>i: Increment()\n\
+   <KeyPress>x: IncrementX()\n\
+   <KeyPress>y: IncrementY()\n\
+   <KeyPress>z: IncrementZ()\n\
    <KeyPress>o: Orientize()";
 
 static XtActionsRec actionsListRubik3D[] =
@@ -127,8 +130,11 @@ static XtActionsRec actionsListRubik3D[] =
 	{"Write", (XtActionProc) WriteRubik},
 	{"Undo", (XtActionProc) UndoRubik},
 	{"Solve", (XtActionProc) SolveRubik},
-	{"Increment", (XtActionProc) IncrementRubik},
 	{"Decrement", (XtActionProc) DecrementRubik},
+	{"Increment", (XtActionProc) IncrementRubik},
+	{"IncrementX", (XtActionProc) IncrementXRubik},
+	{"IncrementY", (XtActionProc) IncrementYRubik},
+	{"IncrementZ", (XtActionProc) IncrementZRubik},
 	{"Orientize", (XtActionProc) OrientizeRubik}
 };
 
@@ -156,12 +162,18 @@ static XtResource resourcesRubik3D[] =
 	 XtOffset(RubikWidget, core.width), XtRString, "250"},
 	{XtNheight, XtCHeight, XtRDimension, sizeof (Dimension),
 	 XtOffset(RubikWidget, core.height), XtRString, "400"},
-	{XtNsize, XtCSize, XtRInt, sizeof (int),
-	 XtOffset(RubikWidget, rubik.size), XtRString, "3"},	/* DEFAULTCUBES */
+	{XtNsizex, XtCSizeX, XtRInt, sizeof (int),
+	 XtOffset(RubikWidget, rubik.sizex), XtRString, "3"},	/* DEFAULTCUBES */
+	{XtNsizey, XtCSizeY, XtRInt, sizeof (int),
+	 XtOffset(RubikWidget, rubik.sizey), XtRString, "3"},	/* DEFAULTCUBES */
+	{XtNsizez, XtCSizeZ, XtRInt, sizeof (int),
+	 XtOffset(RubikWidget, rubik.sizez), XtRString, "3"},	/* DEFAULTCUBES */
 	{XtNorient, XtCOrient, XtRBoolean, sizeof (Boolean),
 	 XtOffset(RubikWidget, rubik.orient), XtRString, "FALSE"},	/* DEFAULTORIENT */
 	{XtNmono, XtCMono, XtRBoolean, sizeof (Boolean),
 	 XtOffset(RubikWidget, rubik.mono), XtRString, "FALSE"},
+	{XtNreverse, XtCReverse, XtRBoolean, sizeof (Boolean),
+	 XtOffset(RubikWidget, rubik.reverse), XtRString, "FALSE"},
 	{XtNface, XtCFace, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.currentFace), XtRString, "-1"},
 	{XtNpos, XtCPos, XtRInt, sizeof (int),
@@ -237,9 +249,9 @@ static RowNext rotateToRow[MAXFACES] =	/*CW to min face */
 };
 
 static void
-InitializeRubik3D(Widget request, Widget new)
+InitializeRubik3D(Widget request, Widget renew)
 {
-	Rubik3DWidget w = (Rubik3DWidget) new;
+	Rubik3DWidget w = (Rubik3DWidget) renew;
 
 	w->rubik.dim = 3;
 	ResizeRubik3D(w);
@@ -248,55 +260,55 @@ InitializeRubik3D(Widget request, Widget new)
 static void
 ResizeRubik3D(Rubik3DWidget w)
 {
-	XPoint      tempSize;
+	/* XPoint      tempSize; */
 
 	w->rubik.delta = 4;
 	w->rubik.vertical = (w->core.height >= w->core.width);
+	/* Unlike the 2d implementation... lets have variable size cubies and
+	   static sized cube */
 	if (w->rubik.vertical) {
-		tempSize.y = w->core.height / MAXVIEWS;
-		tempSize.x = w->core.width;
-		if (tempSize.x >= DIVIDE(tempSize.y)) {
-			w->rubik3d.cubeSize.y = MAX((tempSize.y - 3 * w->rubik.delta) /
-				(2 * w->rubik.size) - w->rubik.delta - 2, 0);
-			w->rubik3d.cubeSize.x = DIVIDE(w->rubik3d.cubeSize.y);
+		w->rubik3d.viewSize.y = w->core.height / MAXVIEWS;
+		w->rubik3d.viewSize.x = w->core.width;
+		if (w->rubik3d.viewSize.x >= DIVIDE(w->rubik3d.viewSize.y)) {
+			w->rubik3d.viewSize.y = MAX(w->rubik3d.viewSize.y, 0);
+			w->rubik3d.viewSize.x = DIVIDE(w->rubik3d.viewSize.y);
 		} else {
-			w->rubik3d.cubeSize.x = MAX((tempSize.x - 2 * w->rubik.delta - 7) /
-				    (2 * w->rubik.size) - w->rubik.delta, 0);
-			w->rubik3d.cubeSize.y = MULTIPLY(w->rubik3d.cubeSize.x);
+			w->rubik3d.viewSize.x = MAX(w->rubik3d.viewSize.x, 0);
+			w->rubik3d.viewSize.y = MULTIPLY(w->rubik3d.viewSize.x);
 		}
-		w->rubik3d.cubeDiagonal = w->rubik3d.cubeSize.x / 2;
-		w->rubik3d.faceSize.x = w->rubik.size *
-			(w->rubik3d.cubeSize.x + w->rubik.delta) + w->rubik.delta + 1;
-		w->rubik3d.faceSize.y = w->rubik.size *
-			(w->rubik3d.cubeSize.y + w->rubik.delta) + w->rubik.delta + 1;
+		w->rubik3d.faceSize.x = w->rubik3d.viewSize.x / 2 - 2;
+		w->rubik3d.faceSize.y = w->rubik3d.viewSize.y / 2 - 2;
 		w->rubik3d.faceDiagonal = w->rubik3d.faceSize.x / 2;
-		w->rubik3d.viewSize.x = 2 * w->rubik3d.faceSize.x + 3;
-		w->rubik3d.viewSize.y = 2 * w->rubik3d.faceSize.y + 3;
-		w->rubik.puzzleSize.x = w->rubik3d.viewSize.x + 1;
-		w->rubik.puzzleSize.y = MAXVIEWS * w->rubik3d.viewSize.y + 1;
+		w->rubik.puzzleSize.x = w->rubik3d.viewSize.x;
+		w->rubik.puzzleSize.y = MAXVIEWS * w->rubik3d.viewSize.y;
 	} else {
-		tempSize.x = w->core.width / MAXVIEWS;
-		tempSize.y = w->core.height;
-		if (tempSize.y >= DIVIDE(tempSize.x)) {
-			w->rubik3d.cubeSize.x = MAX((tempSize.x - 3 * w->rubik.delta) /
-				(2 * w->rubik.size) - w->rubik.delta - 2, 0);
-			w->rubik3d.cubeSize.y = DIVIDE(w->rubik3d.cubeSize.x);
+		w->rubik3d.viewSize.x = w->core.width / MAXVIEWS;
+		w->rubik3d.viewSize.y = w->core.height;
+		if (w->rubik3d.viewSize.y >= DIVIDE(w->rubik3d.viewSize.x)) {
+			w->rubik3d.viewSize.x = MAX(w->rubik3d.viewSize.x, 0);
+			w->rubik3d.viewSize.y = DIVIDE(w->rubik3d.viewSize.x);
 		} else {
-			w->rubik3d.cubeSize.y = MAX((tempSize.y - 2 * w->rubik.delta - 7) /
-				    (2 * w->rubik.size) - w->rubik.delta, 0);
-			w->rubik3d.cubeSize.x = MULTIPLY(w->rubik3d.cubeSize.y);
+			w->rubik3d.viewSize.y = MAX(w->rubik3d.viewSize.y, 0);
+			w->rubik3d.viewSize.x = MULTIPLY(w->rubik3d.viewSize.y);
 		}
-		w->rubik3d.cubeDiagonal = w->rubik3d.cubeSize.y / 2;
-		w->rubik3d.faceSize.y = w->rubik.size *
-			(w->rubik3d.cubeSize.y + w->rubik.delta) + w->rubik.delta + 1;
-		w->rubik3d.faceSize.x = w->rubik.size *
-			(w->rubik3d.cubeSize.x + w->rubik.delta) + w->rubik.delta + 1;
+		w->rubik3d.faceSize.x = w->rubik3d.viewSize.x / 2 - 2;
+		w->rubik3d.faceSize.y = w->rubik3d.viewSize.y / 2 - 2;
 		w->rubik3d.faceDiagonal = w->rubik3d.faceSize.y / 2;
-		w->rubik3d.viewSize.y = 2 * w->rubik3d.faceSize.y + 3;
-		w->rubik3d.viewSize.x = 2 * w->rubik3d.faceSize.x + 3;
-		w->rubik.puzzleSize.y = w->rubik3d.viewSize.y + 1;
-		w->rubik.puzzleSize.x = MAXVIEWS * w->rubik3d.viewSize.x + 1;
+		w->rubik.puzzleSize.y = w->rubik3d.viewSize.y;
+		w->rubik.puzzleSize.x = MAXVIEWS * w->rubik3d.viewSize.x;
 	}
+	w->rubik3d.cubeSizex.x = MAX(w->rubik3d.faceSize.x /
+				     w->rubik.sizex - w->rubik.delta, 0);
+	w->rubik3d.cubeSizey.x = MAX(w->rubik3d.faceSize.x /
+				     w->rubik.sizey - w->rubik.delta, 0);
+	w->rubik3d.cubeSizez.x = MAX(w->rubik3d.faceSize.x /
+				     w->rubik.sizez - w->rubik.delta, 0);
+	w->rubik3d.cubeSizex.y = MAX(w->rubik3d.faceSize.y /
+				     w->rubik.sizex - w->rubik.delta, 0);
+	w->rubik3d.cubeSizey.y = MAX(w->rubik3d.faceSize.y /
+				     w->rubik.sizey - w->rubik.delta, 0);
+	w->rubik3d.cubeSizez.y = MAX(w->rubik3d.faceSize.y /
+				     w->rubik.sizez - w->rubik.delta, 0);
 	w->rubik.puzzleOffset.x = ((int) w->core.width - w->rubik.puzzleSize.x) / 2;
 	w->rubik.puzzleOffset.y = ((int) w->core.height - w->rubik.puzzleSize.y) /
 		2;
@@ -304,31 +316,40 @@ ResizeRubik3D(Rubik3DWidget w)
 }
 
 static void
-ExposeRubik3D(Widget new, XEvent * event, Region region)
-		 /* Not used */
+ExposeRubik3D(Widget renew, XEvent * event, Region region)
 {
-	Rubik3DWidget w = (Rubik3DWidget) new;
+	Rubik3DWidget w = (Rubik3DWidget) renew;
 
 	if (w->core.visible) {
+		if (w->rubik.reverse)
+			XFillRectangle(XtDisplay(w), XtWindow(w),
+				       w->rubik.inverseGC, 0, 0, w->core.width, w->core.height);
 		DrawFrame(w, w->rubik.puzzleGC);
 		DrawAllPolyhedrons((RubikWidget) w);
 	}
 }
 
 static      Boolean
-SetValuesRubik3D(Widget current, Widget request, Widget new)
+SetValuesRubik3D(Widget current, Widget request, Widget renew)
 {
-	Rubik3DWidget c = (Rubik3DWidget) current, w = (Rubik3DWidget) new;
-	Boolean     redraw = FALSE;
+	Rubik3DWidget c = (Rubik3DWidget) current, w = (Rubik3DWidget) renew;
+	Boolean     redraw = False;
 
-	if (w->rubik.size != c->rubik.size) {
+	if (w->rubik.sizex != c->rubik.sizex ||
+	    w->rubik.sizey != c->rubik.sizey ||
+	    w->rubik.sizez != c->rubik.sizez) {
 		ResetPolyhedrons((RubikWidget) w);
 		ResizeRubik3D(w);
-		redraw = TRUE;
+		redraw = True;
 	}
-	if (w->rubik3d.cubeSize.x != c->rubik3d.cubeSize.x) {
+	if (w->rubik3d.cubeSizex.x != c->rubik3d.cubeSizex.x ||
+	    w->rubik3d.cubeSizey.x != c->rubik3d.cubeSizey.x ||
+	    w->rubik3d.cubeSizez.x != c->rubik3d.cubeSizez.x ||
+	    w->rubik3d.cubeSizex.y != c->rubik3d.cubeSizex.y ||
+	    w->rubik3d.cubeSizey.y != c->rubik3d.cubeSizey.y ||
+	    w->rubik3d.cubeSizez.y != c->rubik3d.cubeSizez.y) {
 		ResizeRubik3D(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	return (redraw);
 }
@@ -392,7 +413,7 @@ MoveRubik3DBr(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
 static void
 ResizePolyhedrons(Rubik3DWidget w)
 {
-	int         face, orient, side, corner;
+	int         face, orient, side, corner, sizex, sizey;
 	XPoint      subcubeLoc3D[MAXFACES][MAXORIENT];
 	XPoint      diamondLoc3D[MAXFACES][MAXORIENT];
 	XPoint      subdiamondLoc3D[MAXFACES][MAXORIENT];
@@ -489,63 +510,111 @@ ResizePolyhedrons(Rubik3DWidget w)
 		cubeLoc3D[face][0].y = faceLoc3D[face][0].y;
 		subcubeLoc3D[face][0].x = faceLoc3D[face][0].x;
 		subcubeLoc3D[face][0].y = faceLoc3D[face][0].y;
+		if (face == 0 || face == 4) {
+			sizex = w->rubik.sizex;
+			sizey = w->rubik.sizez;
+		} else if (face == 1 || face == 3) {
+			sizex = w->rubik.sizez;
+			sizey = w->rubik.sizey;
+		} else {	/* if (face == 2 || face == 5) */
+			sizex = w->rubik.sizex;
+			sizey = w->rubik.sizey;
+		}
 		for (orient = 1; orient < MAXORIENT; orient++) {
-			cubeLoc3D[face][orient].x =
-				(faceLoc3D[face][orient].x - (3 + w->rubik.size) *
+			if (((orient & 1) && (face == 2 || face == 3 || face == 4)) ||
+			    (!(orient & 1) && !(face == 2 || face == 3 || face == 4)))
+				cubeLoc3D[face][orient].x =
+					(faceLoc3D[face][orient].x - (3 + sizex) *
 				 w->rubik.delta * faceLoc3D[face][orient].x /
-				 w->rubik3d.faceSize.x) / w->rubik.size;
-			cubeLoc3D[face][orient].y =
-				(faceLoc3D[face][orient].y - (3 + w->rubik.size) *
-				 w->rubik.delta * faceLoc3D[face][orient].y /
-				 w->rubik3d.faceSize.y) / w->rubik.size;
-			subcubeLoc3D[face][orient].x =
-				(faceLoc3D[face][orient].x - (5 + w->rubik.size) *
+					 w->rubik3d.faceSize.x) / sizex;
+			else
+				cubeLoc3D[face][orient].x =
+					(faceLoc3D[face][orient].x - (3 + sizey) *
 				 w->rubik.delta * faceLoc3D[face][orient].x /
-				 w->rubik3d.faceSize.x) / (2 * w->rubik.size);
-			subcubeLoc3D[face][orient].y =
-				(faceLoc3D[face][orient].y - (5 + w->rubik.size) *
+					 w->rubik3d.faceSize.x) / sizey;
+			if (((orient & 1) && !(face == 2 || face == 3 || face == 4)) ||
+			    (!(orient & 1) && (face == 2 || face == 3 || face == 4)))
+				cubeLoc3D[face][orient].y =
+					(faceLoc3D[face][orient].y - (3 + sizey) *
 				 w->rubik.delta * faceLoc3D[face][orient].y /
-				 w->rubik3d.faceSize.y) / (2 * w->rubik.size);
+					 w->rubik3d.faceSize.y) / sizey;
+			else
+				cubeLoc3D[face][orient].y =
+					(faceLoc3D[face][orient].y - (3 + sizex) *
+				 w->rubik.delta * faceLoc3D[face][orient].y /
+					 w->rubik3d.faceSize.y) / sizex;
+			if (((orient & 1) && (face == 2 || face == 3 || face == 4)) ||
+			    (!(orient & 1) && !(face == 2 || face == 3 || face == 4)))
+				subcubeLoc3D[face][orient].x =
+					(faceLoc3D[face][orient].x - (5 + sizex) *
+				 w->rubik.delta * faceLoc3D[face][orient].x /
+					 w->rubik3d.faceSize.x) / (2 * sizex);
+			else
+				subcubeLoc3D[face][orient].x =
+					(faceLoc3D[face][orient].x - (5 + sizey) *
+				 w->rubik.delta * faceLoc3D[face][orient].x /
+					 w->rubik3d.faceSize.x) / (2 * sizey);
+			if (((orient & 1) && !(face == 2 || face == 3 || face == 4)) ||
+			    (!(orient & 1) && (face == 2 || face == 3 || face == 4)))
+				subcubeLoc3D[face][orient].y =
+					(faceLoc3D[face][orient].y - (5 + sizey) *
+				 w->rubik.delta * faceLoc3D[face][orient].y /
+					 w->rubik3d.faceSize.y) / (2 * sizey);
+			else
+				subcubeLoc3D[face][orient].y =
+					(faceLoc3D[face][orient].y - (5 + sizex) *
+				 w->rubik.delta * faceLoc3D[face][orient].y /
+					 w->rubik3d.faceSize.y) / (2 * sizex);
 		}
 		cubeLoc3D[face][MAXORIENT].x = -cubeLoc3D[face][1].x -
 			cubeLoc3D[face][2].x - cubeLoc3D[face][3].x;
 		cubeLoc3D[face][MAXORIENT].y = -cubeLoc3D[face][1].y -
 			cubeLoc3D[face][2].y - cubeLoc3D[face][3].y;
 	}
-	w->rubik3d.cubeSize.x = (w->rubik3d.faceSize.x - w->rubik.delta) /
-		w->rubik.size - w->rubik.delta;
-	w->rubik3d.cubeDiagonal = (w->rubik3d.faceDiagonal - w->rubik.delta) /
-		w->rubik.size - w->rubik.delta;
-	w->rubik3d.cubeSize.y = (w->rubik3d.faceSize.y - w->rubik.delta) /
-		w->rubik.size - w->rubik.delta;
-	w->rubik3d.cubeDiag = (w->rubik3d.faceDiagonal + w->rubik.delta) /
-		w->rubik.size + w->rubik.delta;
+	w->rubik3d.cubeSizex.x = MAX(w->rubik3d.faceSize.x /
+				     w->rubik.sizex - w->rubik.delta, 0);
+	w->rubik3d.cubeSizey.x = MAX(w->rubik3d.faceSize.x /
+				     w->rubik.sizey - w->rubik.delta, 0);
+	w->rubik3d.cubeSizez.x = MAX(w->rubik3d.faceSize.x /
+				     w->rubik.sizez - w->rubik.delta, 0);
+	w->rubik3d.cubeSizex.y = MAX(w->rubik3d.faceSize.y /
+				     w->rubik.sizex - w->rubik.delta, 0);
+	w->rubik3d.cubeSizey.y = MAX(w->rubik3d.faceSize.y /
+				     w->rubik.sizey - w->rubik.delta, 0);
+	w->rubik3d.cubeSizez.y = MAX(w->rubik3d.faceSize.y /
+				     w->rubik.sizez - w->rubik.delta, 0);
+	w->rubik3d.cubeDiagonalx = (w->rubik3d.faceDiagonal - w->rubik.delta) /
+		w->rubik.sizex - w->rubik.delta;
+	w->rubik3d.cubeDiagonaly = (w->rubik3d.faceDiagonal - w->rubik.delta) /
+		w->rubik.sizey - w->rubik.delta;
+	w->rubik3d.cubeDiagonalz = (w->rubik3d.faceDiagonal - w->rubik.delta) /
+		w->rubik.sizez - w->rubik.delta;
 	if (w->rubik.vertical) {
-		letter3DList[0].x = w->rubik3d.cubeSize.x / 4;
-		letter3DList[0].y = -w->rubik3d.cubeSize.y / 2;
-		letter3DList[1].x = -w->rubik3d.cubeDiagonal;
+		letter3DList[0].x = w->rubik3d.cubeSizex.x / 4;
+		letter3DList[0].y = -w->rubik3d.cubeSizez.y / 2;
+		letter3DList[1].x = -w->rubik3d.cubeDiagonalz;
 		letter3DList[1].y = 0;
-		letter3DList[2].x = w->rubik3d.cubeSize.x / 4;
-		letter3DList[2].y = w->rubik3d.cubeSize.y / 2;
-		letter3DList[3].x = w->rubik3d.cubeDiagonal;
+		letter3DList[2].x = w->rubik3d.cubeSizex.x / 4;
+		letter3DList[2].y = w->rubik3d.cubeSizey.y / 2;
+		letter3DList[3].x = w->rubik3d.cubeDiagonalz;
 		letter3DList[3].y = 0;
-		letter3DList[4].x = -w->rubik3d.cubeSize.x / 4;
-		letter3DList[4].y = -w->rubik3d.cubeSize.y / 2;
-		letter3DList[5].x = -w->rubik3d.cubeSize.x / 4;
-		letter3DList[5].y = w->rubik3d.cubeSize.y / 2;
+		letter3DList[4].x = -w->rubik3d.cubeSizex.x / 4;
+		letter3DList[4].y = -w->rubik3d.cubeSizez.y / 2;
+		letter3DList[5].x = -w->rubik3d.cubeSizex.x / 4;
+		letter3DList[5].y = w->rubik3d.cubeSizey.y / 2;
 	} else {
 		letter3DList[0].x = 0;
-		letter3DList[0].y = -w->rubik3d.cubeDiagonal;
-		letter3DList[1].x = -w->rubik3d.cubeSize.x / 2;
-		letter3DList[1].y = w->rubik3d.cubeSize.y / 4;
-		letter3DList[2].x = w->rubik3d.cubeSize.x / 2;
-		letter3DList[2].y = w->rubik3d.cubeSize.y / 4;
-		letter3DList[3].x = -w->rubik3d.cubeSize.x / 2;
-		letter3DList[3].y = -w->rubik3d.cubeSize.y / 4;
+		letter3DList[0].y = -w->rubik3d.cubeDiagonalz;
+		letter3DList[1].x = -w->rubik3d.cubeSizez.x / 2;
+		letter3DList[1].y = w->rubik3d.cubeSizey.y / 4;
+		letter3DList[2].x = w->rubik3d.cubeSizex.x / 2;
+		letter3DList[2].y = w->rubik3d.cubeSizey.y / 4;
+		letter3DList[3].x = -w->rubik3d.cubeSizez.x / 2;
+		letter3DList[3].y = -w->rubik3d.cubeSizey.y / 4;
 		letter3DList[4].x = 0;
-		letter3DList[4].y = w->rubik3d.cubeDiagonal;
-		letter3DList[5].x = w->rubik3d.cubeSize.x / 2;
-		letter3DList[5].y = -w->rubik3d.cubeSize.y / 4;
+		letter3DList[4].y = w->rubik3d.cubeDiagonalz;
+		letter3DList[5].x = w->rubik3d.cubeSizex.x / 2;
+		letter3DList[5].y = -w->rubik3d.cubeSizey.y / 4;
 	}
 	/* The following figures out where to put the orient lines */
 	for (face = 0; face < MAXFACES; face++) {
@@ -591,10 +660,10 @@ ResizePolyhedrons(Rubik3DWidget w)
 	}
 }
 
-int
+Boolean
 SelectPolyhedrons3D(Rubik3DWidget w, int x, int y, int *face, int *position)
 {
-	int         u, v, front, tl, ur, ul, i, j;
+	int         u, v, front, tl, ur, ul, i, j, sizeOfRow, sizeOfColumn;
 
 	if (w->rubik.vertical) {
 		x -= w->rubik3d.viewMiddle.x;
@@ -641,95 +710,94 @@ SelectPolyhedrons3D(Rubik3DWidget w, int x, int y, int *face, int *position)
 				*face = (ur) ? 4 : 5;
 		}
 	}
+	faceSizes((RubikWidget) w, *face, &sizeOfRow, &sizeOfColumn);
 	if (w->rubik.vertical)
 		switch (*face) {
 			case 0:
 				i = (x - 2 - (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(w->rubik3d.cubeSize.x + w->rubik.delta);
-				j = (y + 2) / (w->rubik3d.cubeSize.y + w->rubik.delta) +
-					w->rubik.size - 1;
+					(w->rubik3d.cubeSizex.x + w->rubik.delta);
+				j = (y + 2) / (w->rubik3d.cubeSizez.y + w->rubik.delta) +
+					sizeOfColumn - 1;
 				break;
 			case 1:
 				i = (x + 4 + (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta)) + w->rubik.size - 1;
+					(2 * (w->rubik3d.cubeDiagonalz + w->rubik.delta)) + sizeOfRow - 1;
 				j = (-x - 6 + (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta));
+					(2 * (w->rubik3d.cubeDiagonaly + w->rubik.delta));
 				break;
 			case 2:
 				i = (x - 4 + (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(w->rubik3d.cubeSize.x + w->rubik.delta);
-				j = (y - 4) / (w->rubik3d.cubeSize.y + w->rubik.delta);
+					(w->rubik3d.cubeSizex.x + w->rubik.delta);
+				j = (y - 4) / (w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			case 3:
 				i = (-x + 5 + (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta)) + w->rubik.size - 1;
+					(2 * (w->rubik3d.cubeDiagonalz + w->rubik.delta)) + sizeOfRow - 1;
 				j = (-x + 7 - (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta)) + w->rubik.size - 1;
+					(2 * (w->rubik3d.cubeDiagonaly + w->rubik.delta)) + sizeOfColumn - 1;
 				break;
 			case 4:
 				i = (x + (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(w->rubik3d.cubeSize.x + w->rubik.delta) + w->rubik.size - 1;
-				j = (y - 2) / (w->rubik3d.cubeSize.y + w->rubik.delta) +
-					w->rubik.size - 1;
+					(w->rubik3d.cubeSizex.x + w->rubik.delta) + sizeOfRow - 1;
+				j = (y - 2) / (w->rubik3d.cubeSizez.y + w->rubik.delta) +
+					sizeOfColumn - 1;
 				break;
 			case 5:
 				i = (x + 2 - (y * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.y)) /
-					(w->rubik3d.cubeSize.x + w->rubik.delta) + w->rubik.size - 1;
-				j = (y - 6) / (w->rubik3d.cubeSize.y + w->rubik.delta);
+					(w->rubik3d.cubeSizex.x + w->rubik.delta) + sizeOfRow - 1;
+				j = (y - 6) / (w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			default:
-				return FALSE;
+				return False;
 	} else
 		switch (*face) {
 			case 0:
 				i = (-y - 3 + (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta));
+					(2 * (w->rubik3d.cubeDiagonalx + w->rubik.delta));
 				j = (y + 1 + (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta)) + w->rubik.size - 1;
+					(2 * (w->rubik3d.cubeDiagonalz + w->rubik.delta)) + sizeOfColumn - 1;
 				break;
 			case 1:
-				i = (x + 2) / (w->rubik3d.cubeSize.x + w->rubik.delta) +
-					w->rubik.size - 1;
+				i = (x + 2) / (w->rubik3d.cubeSizez.x + w->rubik.delta) +
+					sizeOfRow - 1;
 				j = (y - 3 - (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(w->rubik3d.cubeSize.y + w->rubik.delta);
+					(w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			case 2:
-				i = (x - 4) / (w->rubik3d.cubeSize.x + w->rubik.delta);
+				i = (x - 4) / (w->rubik3d.cubeSizex.x + w->rubik.delta);
 				j = (y - 6 + (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(w->rubik3d.cubeSize.y + w->rubik.delta);
+					(w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			case 3:
-				i = x / (w->rubik3d.cubeSize.x + w->rubik.delta) +
-					w->rubik.size - 1;
+				i = x / (w->rubik3d.cubeSizez.x + w->rubik.delta) +
+					sizeOfRow - 1;
 				j = (y + (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(w->rubik3d.cubeSize.y + w->rubik.delta) + w->rubik.size - 1;
+					(w->rubik3d.cubeSizey.y + w->rubik.delta) + sizeOfColumn - 1;
 				break;
 			case 4:
 				i = (-y + 9 - (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta)) + w->rubik.size - 1;
+					(2 * (w->rubik3d.cubeDiagonalx + w->rubik.delta)) + sizeOfRow - 1;
 				j = (-y + 7 + (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(2 * (w->rubik3d.cubeDiagonal + w->rubik.delta)) + w->rubik.size - 1;
+					(2 * (w->rubik3d.cubeDiagonalz + w->rubik.delta)) + sizeOfColumn - 1;
 				break;
 			case 5:
-				i = (-x + 7) / (w->rubik3d.cubeSize.x + w->rubik.delta) +
-					w->rubik.size - 1;
+				i = (-x + 7) / (w->rubik3d.cubeSizex.x + w->rubik.delta) +
+					sizeOfRow - 1;
 				j = (-y - 4 + (x * w->rubik3d.faceDiagonal / w->rubik3d.faceSize.x)) /
-					(w->rubik3d.cubeSize.y + w->rubik.delta);
+					(w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			default:
-				return FALSE;
+				return False;
 		}
-	if (i < 0 || j < 0 || i >= w->rubik.size || j >= w->rubik.size)
-		return FALSE;
-	*position = j * w->rubik.size + i;
-	return TRUE;
+	if (i < 0 || j < 0 || i >= sizeOfRow || j >= sizeOfColumn)
+		return False;
+	*position = j * sizeOfRow + i;
+	return True;
 }
 
-int
+Boolean
 NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 {
-	int         i, j;
-
 	switch (*direction) {
 		case TOP:
 		case RIGHT:
@@ -737,10 +805,10 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 		case LEFT:
 			if (w->rubik.vertical) {
 				if (*face == 1 || *face == 3)
-					return FALSE;
+					return False;
 			} else {
 				if (*face == 0 || *face == 4)
-					return FALSE;
+					return False;
 				if (*face == 5)
 					*direction = (*direction + 2) % MAXORIENT;
 			}
@@ -751,14 +819,14 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 		case TR:
 			if (w->rubik.vertical) {
 				if (*face == 0 || *face == 5)
-					return FALSE;
+					return False;
 				else if (*face == 1 || *face == 2 || *face == 4)
 					*direction = TOP;
 				else	/* (*face == 3) */
 					*direction = LEFT;
 			} else {
 				if (*face == 1 || *face == 5)
-					return FALSE;
+					return False;
 				else if (*face == 0 || *face == 2 || *face == 3)
 					*direction = RIGHT;
 				else	/* (*face == 4) */
@@ -768,7 +836,7 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 		case BR:
 			if (w->rubik.vertical) {
 				if (*face == 2 || *face == 4)
-					return FALSE;
+					return False;
 				else if (*face == 0 || *face == 5)
 					*direction = BOTTOM;
 				else if (*face == 1)
@@ -777,7 +845,7 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 					*direction = TOP;
 			} else {
 				if (*face == 2 || *face == 3)
-					return FALSE;
+					return False;
 				else if (*face == 4 || *face == 5)
 					*direction = LEFT;
 				else if (*face == 0)
@@ -789,14 +857,14 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 		case BL:
 			if (w->rubik.vertical) {
 				if (*face == 0 || *face == 5)
-					return FALSE;
+					return False;
 				else if (*face == 1 || *face == 2 || *face == 4)
 					*direction = BOTTOM;
 				else	/* (*face == 3) */
 					*direction = RIGHT;
 			} else {
 				if (*face == 1 || *face == 5)
-					return FALSE;
+					return False;
 				else if (*face == 0 || *face == 2 || *face == 3)
 					*direction = LEFT;
 				else	/* (*face == 4) */
@@ -806,7 +874,7 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 		case TL:
 			if (w->rubik.vertical) {
 				if (*face == 2 || *face == 4)
-					return FALSE;
+					return False;
 				else if (*face == 0 || *face == 5)
 					*direction = TOP;
 				else if (*face == 1)
@@ -815,7 +883,7 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 					*direction = BOTTOM;
 			} else {
 				if (*face == 2 || *face == 3)
-					return FALSE;
+					return False;
 				else if (*face == 4 || *face == 5)
 					*direction = RIGHT;
 				else if (*face == 0)
@@ -825,22 +893,28 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 			}
 			break;
 		default:
-			return FALSE;
+			return False;
 	}
-	/*cubeDiag = (w->rubik3d.faceDiagonal + w->rubik.delta) / w->rubik.size +
-	   w->rubik.delta; */
 	/* Remap to row movement */
 	if (*direction == CW || *direction == CCW) {
+		int         i, j, sizeOfRow, sizeOfColumn, newFace;
+
+		newFace = rotateToRow[*face].face;
+		faceSizes((RubikWidget) w, newFace, &sizeOfRow, &sizeOfColumn);
 		*direction = (*direction == CCW) ?
 			(rotateToRow[*face].direction + 2) % MAXORIENT :
 			rotateToRow[*face].direction;
-		i = j = (rotateToRow[*face].sideFace == LEFT ||
-			 rotateToRow[*face].sideFace == BOTTOM) ?
-			w->rubik.size - 1 : 0;
-		*face = rotateToRow[*face].face;
-		*position = j * w->rubik.size + i;
+		if (rotateToRow[*face].sideFace == LEFT ||
+		    rotateToRow[*face].sideFace == BOTTOM) {
+			i = sizeOfRow - 1;
+			j = sizeOfColumn - 1;
+		} else {
+			i = j = 0;
+		}
+		*face = newFace;
+		*position = j * sizeOfRow + i;
 	}
-	return TRUE;
+	return True;
 }
 
 static void
@@ -881,74 +955,86 @@ DrawFrame(Rubik3DWidget w, GC gc)
 void
 DrawSquare3D(Rubik3DWidget w, int face, int position, int offset)
 {
-	int         x, y, dx, dy, i, j;
+	GC          faceGC, borderGC;
+	int         x, y, dx, dy, i, j, sizeOfRow;
 
-	i = position % w->rubik.size;
-	j = position / w->rubik.size;
+	sizeOfRow = sizeRow((RubikWidget) w, face);
+	i = position % sizeOfRow;
+	j = position / sizeOfRow;
 	MapTo3D(w, face, i, j, &x, &y);
 	CubeOffset3D(w, face, x, y, &dx, &dy);
 	cubeLoc3D[face][0].x = dx;
 	cubeLoc3D[face][0].y = dy;
 	if (offset) {
-		XFillPolygon(XtDisplay(w), XtWindow(w),
-			     w->rubik.borderGC,
-		      cubeLoc3D[face], MAXORIENT, Convex, CoordModePrevious);
-		XDrawLines(XtDisplay(w), XtWindow(w),
-		      w->rubik.faceGC[(int) w->rubik.cubeLoc[face][position].face],
-			   cubeLoc3D[face], 5, CoordModePrevious);
+		borderGC = w->rubik.faceGC[(int) w->rubik.cubeLoc[face][position].face];
+		if (w->rubik.depth < 2 || w->rubik.mono) {
+			faceGC = w->rubik.inverseGC;
+		} else {
+			faceGC = w->rubik.borderGC;
+		}
 	} else {
-		XFillPolygon(XtDisplay(w), XtWindow(w),
-		      w->rubik.faceGC[(int) w->rubik.cubeLoc[face][position].face],
-		      cubeLoc3D[face], MAXORIENT, Convex, CoordModePrevious);
-		XDrawLines(XtDisplay(w), XtWindow(w),
-		   w->rubik.borderGC, cubeLoc3D[face], 5, CoordModePrevious);
+		faceGC = w->rubik.faceGC[(int) w->rubik.cubeLoc[face][position].face];
+		borderGC = w->rubik.borderGC;
 	}
-	if (w->rubik.depth == 1 || w->rubik.mono) {
+	XFillPolygon(XtDisplay(w), XtWindow(w),
+	      faceGC, cubeLoc3D[face], MAXORIENT, Convex, CoordModePrevious);
+	XDrawLines(XtDisplay(w), XtWindow(w),
+		   borderGC, cubeLoc3D[face], 5, CoordModePrevious);
+	if (w->rubik.depth < 2 || w->rubik.mono) {
 		int         letterX, letterY;
 		char        buf[2];
 
 		(void) sprintf(buf, "%c",
-		w->rubik.faceName[(int) w->rubik.cubeLoc[face][position].face][0]);
+			       w->rubik.faceName[(int) w->rubik.cubeLoc[face][position].face][0]);
 		letterX = dx + letter3DList[face].x + w->rubik.letterOffset.x;
 		letterY = dy + letter3DList[face].y + w->rubik.letterOffset.y;
-		XDrawString(XtDisplay(w), XtWindow(w), w->rubik.inverseGC,
+		if (offset) {
+			borderGC = w->rubik.borderGC;
+		} else {
+			borderGC = w->rubik.inverseGC;
+		}
+		XDrawString(XtDisplay(w), XtWindow(w), borderGC,
 			    letterX, letterY, buf, 1);
 	}
-	if (w->rubik.orient)
-		XDrawLine(XtDisplay(w), XtWindow(w), w->rubik.inverseGC,
+	if (w->rubik.orient) {
+		XDrawLine(XtDisplay(w), XtWindow(w), borderGC,
 			  dx + orient3DList[face][(int) w->rubik.cubeLoc[face][position].rotation][0].x,
 			  dy + orient3DList[face][(int) w->rubik.cubeLoc[face][position].rotation][0].y,
 			  dx + orient3DList[face][(int) w->rubik.cubeLoc[face][position].rotation][1].x,
 			  dy + orient3DList[face][(int) w->rubik.cubeLoc[face][position].rotation][1].y);
+	}
 }
 
 static void
 MapTo3D(Rubik3DWidget w, int face, int i, int j, int *x, int *y)
 {
+	int         sizeOfRow, sizeOfColumn;
+
+	faceSizes((RubikWidget) w, face, &sizeOfRow, &sizeOfColumn);
 	switch (face) {
 		case 0:
-			*x = w->rubik.size - 1 - j;
+			*x = w->rubik.sizez - 1 - j;
 			*y = i;
 			break;
 		case 1:
 			*x = j;
-			*y = w->rubik.size - 1 - i;
+			*y = w->rubik.sizez - 1 - i;
 			break;
 		case 2:
 			*x = i;
 			*y = j;
 			break;
 		case 3:
-			*x = w->rubik.size - 1 - i;
-			*y = w->rubik.size - 1 - j;
+			*x = w->rubik.sizez - 1 - i;
+			*y = w->rubik.sizey - 1 - j;
 			break;
 		case 4:
-			*x = w->rubik.size - 1 - i;
-			*y = w->rubik.size - 1 - j;
+			*x = w->rubik.sizex - 1 - i;
+			*y = w->rubik.sizez - 1 - j;
 			break;
 		case 5:
 			*x = j;
-			*y = w->rubik.size - 1 - i;
+			*y = w->rubik.sizex - 1 - i;
 			break;
 		default:
 			(void) printf("MapTo3D: face %d\n", face);
@@ -1027,50 +1113,50 @@ CubeOffset3D(Rubik3DWidget w, int face, int x, int y, int *dx, int *dy)
 		switch (face) {
 			case 0:
 				*dx = w->rubik3d.viewMiddle.x + w->rubik.delta - 1 +
-					y * (w->rubik3d.cubeSize.x + w->rubik.delta) -
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					y * (w->rubik3d.cubeSizex.x + w->rubik.delta) -
+					x * (w->rubik3d.cubeDiagonalz + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y - w->rubik.delta - 2 -
-					x * (w->rubik3d.cubeSize.y + w->rubik.delta);
+					x * (w->rubik3d.cubeSizez.y + w->rubik.delta);
 				break;
 			case 1:
 				*dx = w->rubik3d.viewMiddle.x - 2 * w->rubik.delta -
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta) -
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					x * (w->rubik3d.cubeDiagonaly + w->rubik.delta) -
+					y * (w->rubik3d.cubeDiagonalz + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y +
-					x * (w->rubik3d.cubeSize.y + w->rubik.delta) -
-					y * (w->rubik3d.cubeSize.y + w->rubik.delta);
+					x * (w->rubik3d.cubeSizey.y + w->rubik.delta) -
+					y * (w->rubik3d.cubeSizez.y + w->rubik.delta);
 				break;
 			case 2:
 				*dx = w->rubik3d.viewMiddle.x + w->rubik.delta - 1 +
-					x * (w->rubik3d.cubeSize.x + w->rubik.delta) -
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					x * (w->rubik3d.cubeSizex.x + w->rubik.delta) -
+					y * (w->rubik3d.cubeDiagonaly + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y + 2 * w->rubik.delta - 1 +
-					y * (w->rubik3d.cubeSize.y + w->rubik.delta);
+					y * (w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			case 3:
 				*dx = w->rubik3d.viewMiddle.x + 2 * w->rubik.delta +
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta) +
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					x * (w->rubik3d.cubeDiagonalz + w->rubik.delta) +
+					y * (w->rubik3d.cubeDiagonaly + w->rubik.delta);
 				*dy = w->rubik3d.viewSize.y + w->rubik3d.viewMiddle.y +
 					w->rubik.delta - 1 -
-					x * (w->rubik3d.cubeSize.y + w->rubik.delta) +
-					y * (w->rubik3d.cubeSize.y + w->rubik.delta);
+					x * (w->rubik3d.cubeSizez.y + w->rubik.delta) +
+					y * (w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			case 4:
 				*dx = w->rubik3d.viewMiddle.x - w->rubik.delta + 1 -
-					x * (w->rubik3d.cubeSize.x + w->rubik.delta) +
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					x * (w->rubik3d.cubeSizex.x + w->rubik.delta) +
+					y * (w->rubik3d.cubeDiagonalz + w->rubik.delta);
 				*dy = w->rubik3d.viewSize.y + w->rubik3d.viewMiddle.y -
 					w->rubik.delta + 1 -
-					y * (w->rubik3d.cubeSize.y + w->rubik.delta);
+					y * (w->rubik3d.cubeSizez.y + w->rubik.delta);
 				break;
 			case 5:
 				*dx = w->rubik3d.viewMiddle.x - 2 -
-					y * (w->rubik3d.cubeSize.x + w->rubik.delta) +
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					y * (w->rubik3d.cubeSizex.x + w->rubik.delta) +
+					x * (w->rubik3d.cubeDiagonaly + w->rubik.delta);
 				*dy = w->rubik3d.viewSize.y + w->rubik3d.viewMiddle.y +
 					2 * w->rubik.delta + 2 +
-					x * (w->rubik3d.cubeSize.y + w->rubik.delta);
+					x * (w->rubik3d.cubeSizey.y + w->rubik.delta);
 				break;
 			default:
 				(void) printf("CubeOffset3D: face %d\n", face);
@@ -1078,50 +1164,50 @@ CubeOffset3D(Rubik3DWidget w, int face, int x, int y, int *dx, int *dy)
 		switch (face) {
 			case 0:
 				*dx = w->rubik3d.viewMiddle.x +
-					y * (w->rubik3d.cubeSize.x + w->rubik.delta) -
-					x * (w->rubik3d.cubeSize.x + w->rubik.delta);
+					y * (w->rubik3d.cubeSizex.x + w->rubik.delta) -
+					x * (w->rubik3d.cubeSizez.x + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y - 2 * w->rubik.delta + 1 -
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta) -
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					y * (w->rubik3d.cubeDiagonalx + w->rubik.delta) -
+					x * (w->rubik3d.cubeDiagonalz + w->rubik.delta);
 				break;
 			case 1:
 				*dx = w->rubik3d.viewMiddle.x - w->rubik.delta - 2 -
-					y * (w->rubik3d.cubeSize.x + w->rubik.delta);
+					y * (w->rubik3d.cubeSizez.x + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y + w->rubik.delta +
-					x * (w->rubik3d.cubeSize.y + w->rubik.delta) -
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					x * (w->rubik3d.cubeSizey.y + w->rubik.delta) -
+					y * (w->rubik3d.cubeDiagonalz + w->rubik.delta);
 				break;
 			case 2:
 				*dx = w->rubik3d.viewMiddle.x + 2 * w->rubik.delta - 1 +
-					x * (w->rubik3d.cubeSize.x + w->rubik.delta);
+					x * (w->rubik3d.cubeSizex.x + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y + w->rubik.delta +
-					y * (w->rubik3d.cubeSize.y + w->rubik.delta) -
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					y * (w->rubik3d.cubeSizey.y + w->rubik.delta) -
+					x * (w->rubik3d.cubeDiagonalx + w->rubik.delta);
 				break;
 			case 3:
 				*dx = w->rubik3d.viewSize.x + w->rubik3d.viewMiddle.x -
 					w->rubik.delta + 1 -
-					x * (w->rubik3d.cubeSize.x + w->rubik.delta);
+					x * (w->rubik3d.cubeSizez.x + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y - w->rubik.delta -
-					y * (w->rubik3d.cubeSize.y + w->rubik.delta) +
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					y * (w->rubik3d.cubeSizey.y + w->rubik.delta) +
+					x * (w->rubik3d.cubeDiagonalz + w->rubik.delta);
 				break;
 			case 4:
 				*dx = w->rubik3d.viewSize.x + w->rubik3d.viewMiddle.x +
 					w->rubik.delta - 1 -
-					y * (w->rubik3d.cubeSize.x + w->rubik.delta) +
-					x * (w->rubik3d.cubeSize.x + w->rubik.delta);
+					y * (w->rubik3d.cubeSizez.x + w->rubik.delta) +
+					x * (w->rubik3d.cubeSizex.x + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y + 2 * w->rubik.delta +
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta) +
-					x * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					y * (w->rubik3d.cubeDiagonalz + w->rubik.delta) +
+					x * (w->rubik3d.cubeDiagonalx + w->rubik.delta);
 				break;
 			case 5:
 				*dx = w->rubik3d.viewSize.x + w->rubik3d.viewMiddle.x +
 					2 * w->rubik.delta + 2 +
-					y * (w->rubik3d.cubeSize.x + w->rubik.delta);
+					y * (w->rubik3d.cubeSizex.x + w->rubik.delta);
 				*dy = w->rubik3d.viewMiddle.y - w->rubik.delta -
-					x * (w->rubik3d.cubeSize.y + w->rubik.delta) +
-					y * (w->rubik3d.cubeDiagonal + w->rubik.delta);
+					x * (w->rubik3d.cubeSizey.y + w->rubik.delta) +
+					y * (w->rubik3d.cubeDiagonalx + w->rubik.delta);
 				break;
 			default:
 				(void) printf("CubeOffset3D: face %d\n", face);
